@@ -1,7 +1,7 @@
 #############################################################
 #  dirflow_module.tcl  
 # -----------------------------------------------------------
-#  Copyright (C) 2003 Tresys Technology, LLC
+#  Copyright (C) 2003-2005 Tresys Technology, LLC
 #  see file 'COPYING' for use and warranty information 
 #
 #  Requires tcl and tk 8.3+, with BWidgets
@@ -20,9 +20,27 @@ namespace eval Apol_Analysis_dirflow {
      	variable combo_attribute
         variable combo_start
         variable list_objs
-    	variable info_button_text "\n\nThis analysis generates the results of a Direct Information Flow analysis beginning from the starting type selected.  The results of the analysis are presented in tree form with the root of the tree being the start point for the analysis.\n\nEach child node in the tree represents a type in the current policy for which there is a direct information flow to or from its parent node.  If 'in' was selected then the information flow is from the child to the parent.  If 'out' was selected then information flows from the parent to the child.\n\nThe results of the analysis may be optionally filtered by object class selection or an end type regular expression.\n\nNOTE: For any given generation, if the parent and the child are the same, you cannot open the child.  This avoids cyclic analyses.\n\nFor additional help on this topic select \"Information Flow Analysis\" from the help menu."
-        variable root_text \
-"\n\nThis tab provides the results of a Direct Information Flow analysis beginning from the starting type selected above.  The results of the analysis are presented in tree form with the root of the tree (this node) being the start point for the analysis.\n\nEach child node in the tree represents a type in the current policy for which there is a direct information flow to or from (depending on your selection above) its parent node.\n\nNOTE: For any given generation, if the parent and the child are the same, you cannot open the child.  This avoids cyclic analyses.\n\n"
+    	variable info_button_text "\n\nThis analysis generates the results of a Direct Information Flow \
+    				  analysis beginning from the starting type selected.  The results of \
+    				  the analysis are presented in tree form with the root of the tree being \
+    				  the start point for the analysis.\n\nEach child node in the tree represents \
+    				  a type in the current policy for which there is a direct information flow \
+    				  to or from its parent node.  If 'in' was selected then the information flow \
+    				  is from the child to the parent.  If 'out' was selected then information \
+    				  flows from the parent to the child.\n\nThe results of the analysis may be \
+    				  optionally filtered by object class selection or an end type regular \
+    				  expression.\n\nNOTE: For any given generation, if the parent and the child \
+    				  are the same, you cannot open the child.  This avoids cyclic analyses.\n\nFor \
+    				  additional help on this topic select \"Information Flow Analysis\" from the \
+    				  help menu."
+        variable root_text  "\n\nThis tab provides the results of a Direct Information Flow analysis beginning \
+        		    from the starting type selected above.  The results of the analysis are presented \
+        		    in tree form with the root of the tree (this node) being the start point for the \
+        		    analysis.\n\nEach child node in the tree represents a type in the current policy \
+        		    for which there is a direct information flow to or from (depending on your selection \
+        		    above) its parent node.\n\nNOTE: For any given generation, if the parent and the child \
+        		    are the same, you cannot open the child.  This avoids cyclic analyses.\n\n"
+        		    
         variable in_button
         variable out_button
         variable either_button
@@ -58,9 +76,6 @@ namespace eval Apol_Analysis_dirflow {
 	variable counters_tag		COUNTERS
 	variable types_tag		TYPE
 	variable disabled_rule_tag     	DISABLE_RULE
-
-    	# Return value to indicate that perm map loaded successfully, but there were warnings
-	variable warning_return_val	"-2"
 	
 ## Within the namespace command for the module, you must call Apol_Analysis::register_analysis_modules,
 ## the first argument is the namespace name of the module, and the second is the
@@ -103,6 +118,13 @@ proc Apol_Analysis_dirflow::get_analysis_info {} {
      	return $Apol_Analysis_dirflow::info_button_text
 } 
 
+# ------------------------------------------------------------------------------
+#  Command Apol_Analysis_dirflow::get_results_raised_tab
+# ------------------------------------------------------------------------------
+proc Apol_Analysis_dirflow::get_results_raised_tab {} {
+     	return $Apol_Analysis_dirflow::dirflow_info_text
+} 
+
 ## Command Apol_Analysis_dirflow::do_analysis is the principal interface command.
 ## The GUI will call this when the module is to perform it's analysis.  The
 ## module should know how to get its own option information (the options
@@ -119,9 +141,8 @@ proc Apol_Analysis_dirflow::do_analysis { results_frame } {
         variable flow_direction
         variable list_objs
         variable objects_sel
-	variable warning_return_val
 	
-        set selected_objects [Apol_Analysis_dirflow::get_selected_objects]
+        set selected_objects [Apol_Analysis_dirflow::get_unselected_objects]
         
         # if a permap is not loaded then load the default permap
         # if an error occurs on open, then skip analysis
@@ -133,7 +154,7 @@ proc Apol_Analysis_dirflow::do_analysis { results_frame } {
 	if { !$map_loaded } {
 	    set rt [catch {Apol_Perms_Map::load_default_perm_map} err]
 	    if { $rt != 0 } {
-		if {$rt == $warning_return_val} {
+		if {$rt == $Apol_Perms_Map::warning_return_val} {
 			tk_messageBox -icon warning -type ok -title "Warning" -message "$err"
 		} else {
 			tk_messageBox -icon error -type ok -title "Error" -message "$err"
@@ -141,7 +162,7 @@ proc Apol_Analysis_dirflow::do_analysis { results_frame } {
 		}
 	    }
 	}
-	
+	update idletasks
      	set rt [catch {set results [apol_DirectInformationFlowAnalysis \
 		$Apol_Analysis_dirflow::start_type \
 		$Apol_Analysis_dirflow::flow_direction \
@@ -164,7 +185,10 @@ proc Apol_Analysis_dirflow::do_analysis { results_frame } {
 		$Apol_Analysis_dirflow::end_type]
 
 	set dirflow_tree [Apol_Analysis_dirflow::create_resultsDisplay $results_frame]
-	set rt [catch {Apol_Analysis_dirflow::create_result_tree_structure $dirflow_tree $results $query_args} err]
+	set rt [catch {Apol_Analysis_dirflow::create_result_tree_structure \
+		$dirflow_tree \
+		$results \
+		$query_args} err]
 	if {$rt != 0} {
 		tk_messageBox -icon error -type ok -title "Error" -message "$err"
 		return -code error
@@ -217,6 +241,7 @@ proc Apol_Analysis_dirflow::open { } {
 proc Apol_Analysis_dirflow::load_query_options { file_channel parentDlg } {
 	
 	set query_options ""
+	set query_options_tmp ""
         while {[eof $file_channel] != 1} {
 		gets $file_channel line
 		set tline [string trim $line]
@@ -224,15 +249,20 @@ proc Apol_Analysis_dirflow::load_query_options { file_channel parentDlg } {
 		if {$tline == "" || [string compare -length 1 $tline "#"] == 0} {
 			continue
 		}
-		set query_options [lappend query_options $tline]
+		set query_options_tmp [lappend query_options_tmp $tline]
 	}
-	if {$query_options == ""} {
+	if {$query_options_tmp == ""} {
 		return -code error "No query parameters were found."
 	}
 	# Re-format the query options list into a string where all elements are seperated
-	# by a single space. Then split this string into a list using the space as the delimeter.	
-	set query_options [split [join $query_options " "]]
-	
+	# by a single space. Then split this string into a list using space and colon characters
+	# as the delimeters.	
+	set query_options_tmp [split [join $query_options_tmp " "] " :"]
+	set query_options [ApolTop::strip_list_of_empty_items $query_options_tmp]
+	if {$query_options == ""} {
+		return -code error "No query parameters were found."
+	}
+		
         Apol_Analysis_dirflow::clear_all_button_press
         # Query option variables
         set Apol_Analysis_dirflow::endtype_sel [lindex $query_options 0]
@@ -294,13 +324,11 @@ proc Apol_Analysis_dirflow::load_query_options { file_channel parentDlg } {
         }
 	# Display a popup with a list of invalid objects
 	if {$invalid_objs != ""} {
+		puts "The following objects do not exist in the currently \
+			loaded policy and were ignored:\n\n"
 		foreach obj $invalid_objs {
-			set objs_str [append objs_str "$obj\n"]	
+			puts "$obj\n"
 		}
-		tk_messageBox -icon warning -type ok -title "Invalid Objects" \
-			-message "The following objects do not exist in the currently \
-			loaded policy and were ignored.\n\n$objs_str" \
-			-parent $parentDlg
 	}
 		        
 	Apol_Analysis_dirflow::config_endtype_state
@@ -494,17 +522,19 @@ proc Apol_Analysis_dirflow::free_results_data {query_options} {
 #################################################################################
 #################################################################################
 proc Apol_Analysis_dirflow::treeSelect {dirflow_tree dirflow_info_text node} {
-# Set the tree selection to the current node.
-	
+	# Set the tree selection to the current node.
 	$dirflow_tree selection set $node
-
         if {$node ==  [$dirflow_tree nodes root]} {
-		Apol_Analysis_dirflow::display_root_type_info $node $dirflow_info_text $dirflow_tree
-	        Apol_Analysis_dirflow::formatInfoText $dirflow_info_text
-		return
+		Apol_Analysis_dirflow::display_root_type_info $node \
+			$dirflow_info_text $dirflow_tree
+	} else {
+		Apol_Analysis_dirflow::render_target_type_data \
+			[$dirflow_tree itemcget $node -data] \
+			$dirflow_info_text $dirflow_tree $node
 	}
-	Apol_Analysis_dirflow::render_target_type_data [$dirflow_tree itemcget $node -data] $dirflow_info_text $dirflow_tree $node
 	Apol_Analysis_dirflow::formatInfoText $dirflow_info_text
+	ApolTop::makeTextBoxReadOnly $dirflow_info_text
+	
 	return 0
 
 }
@@ -530,7 +560,6 @@ proc Apol_Analysis_dirflow::display_root_type_info { source_type dirflow_info_te
     $dirflow_info_text insert end $Apol_Analysis_dirflow::root_text
     $dirflow_info_text tag add ROOT_TEXT $start_idx end
     $dirflow_info_text tag configure ROOT_TEXT -font $ApolTop::text_font 
-    $dirflow_info_text configure -state disabled
 
     return 0
 }
@@ -541,7 +570,6 @@ proc Apol_Analysis_dirflow::render_target_type_data {data dirflow_info_text dirf
         $dirflow_info_text configure -wrap none
 
 	if { $data == "" } {
-	        $dirflow_info_text configure -state disabled
 		return ""
 	}
         set cur_end_type [lindex $data 0]
@@ -606,7 +634,7 @@ proc Apol_Analysis_dirflow::render_target_type_data {data dirflow_info_text dirf
 			set rule [string range $rule [expr $end_link_idx + 1] end]
 			
 			# Only display line number hyperlink if this is not a binary policy.
-			if {$ApolTop::policy_type != $ApolTop::binary_policy_type} {
+			if {![ApolTop::is_binary_policy]} {
 				$dirflow_info_text insert end "\[$lineno\]"
 				Apol_PolicyConf::insertHyperLink $dirflow_info_text "$startIdx wordstart + 1c" "$startIdx wordstart + [expr [string length $lineno] + 1]c"
 			}
@@ -706,7 +734,7 @@ proc Apol_Analysis_dirflow::render_target_type_data {data dirflow_info_text dirf
 			set rule [string range $rule [expr $end_link_idx + 1] end]
 			
 			# Only display line number hyperlink if this is not a binary policy.
-			if {$ApolTop::policy_type != $ApolTop::binary_policy_type} {
+			if {![ApolTop::is_binary_policy]} {
 				$dirflow_info_text insert end "\[$lineno\]"
 				Apol_PolicyConf::insertHyperLink $dirflow_info_text "$startIdx wordstart + 1c" "$startIdx wordstart + [expr [string length $lineno] + 1]c"
 			}
@@ -730,7 +758,6 @@ proc Apol_Analysis_dirflow::render_target_type_data {data dirflow_info_text dirf
 		incr curIdx
 	    }
 	}
-	$dirflow_info_text configure -state disabled
 	return
 }
 
@@ -779,7 +806,7 @@ proc Apol_Analysis_dirflow::create_target_type_nodes { parent dirflow_tree resul
 			set target_name [lindex $results_list $curentIdx]		        	
 			set nextIdx [Apol_Analysis_dirflow::parseList_get_index_next_node $curentIdx $results_list]
 			if {$nextIdx == -1} {
-				return -code error "Error parsing results"
+				return -code error "Error parsing results. See stdout for more information."
 			}
 
 			set target_node "${parent}/${target_name}/"
@@ -839,7 +866,7 @@ proc Apol_Analysis_dirflow::parseList_get_index_next_node { currentIdx results_l
 			incr currentIdx
 		}
         } else {
-        	puts "Invalid flow direction encountered while parsing results."
+        	puts "Invalid flow direction ($direction) encountered while parsing results."
         	return -1
         }
 
@@ -847,12 +874,17 @@ proc Apol_Analysis_dirflow::parseList_get_index_next_node { currentIdx results_l
 }
 
 proc Apol_Analysis_dirflow::create_result_tree_structure { dirflow_tree results_list query_args} {
-        set home_node [Apol_Analysis_dirflow::insert_src_type_node $dirflow_tree $query_args]
-	set rt [catch {Apol_Analysis_dirflow::create_target_type_nodes $home_node $dirflow_tree $results_list} err]
+        set home_node [Apol_Analysis_dirflow::insert_src_type_node $dirflow_tree \
+        	$query_args]
+	set rt [catch {Apol_Analysis_dirflow::create_target_type_nodes $home_node \
+		$dirflow_tree $results_list} err]
 	if {$rt != 0} {
 		return -code error $err
 	}
-	Apol_Analysis_dirflow::treeSelect $Apol_Analysis_dirflow::dirflow_tree $Apol_Analysis_dirflow::dirflow_info_text $home_node
+	Apol_Analysis_dirflow::treeSelect \
+		$Apol_Analysis_dirflow::dirflow_tree \
+		$Apol_Analysis_dirflow::dirflow_info_text \
+		$home_node
         return 0
 }
 
@@ -903,10 +935,14 @@ proc Apol_Analysis_dirflow::create_resultsDisplay { results_frame } {
 		   -redraw 0 -bg white -showlines 1 -padx 0 \
 		   -opencmd  {Apol_Analysis_dirflow::do_child_analysis $Apol_Analysis_dirflow::dirflow_tree}]
 	$sw_tree setwidget $dirflow_tree 
-
+		
 	# info window
-	set dirflow_info_text [text [$sw_info getframe].dirflow_info_text -wrap none -bg white -font $ApolTop::text_font]
+	set dirflow_info_text [text [$sw_info getframe].dirflow_info_text \
+		-wrap none \
+		-bg white \
+		-font $ApolTop::text_font]
 	$sw_info setwidget $dirflow_info_text
+	bind $dirflow_info_text <Enter> {focus %W}
 	
 	pack $pw -fill both -expand yes -anchor nw 
 	pack $frm_tree -fill both -expand yes -anchor nw
@@ -914,30 +950,34 @@ proc Apol_Analysis_dirflow::create_resultsDisplay { results_frame } {
 	pack $sw_tree -fill both -expand yes
 	pack $sw_info -fill both -expand yes 
 	
-	$dirflow_tree bindText  <ButtonPress-1> {Apol_Analysis_dirflow::treeSelect \
-		$Apol_Analysis_dirflow::dirflow_tree $Apol_Analysis_dirflow::dirflow_info_text}
+	$dirflow_tree bindText  <ButtonPress-1> {
+		Apol_Analysis_dirflow::treeSelect \
+		$Apol_Analysis_dirflow::dirflow_tree \
+		$Apol_Analysis_dirflow::dirflow_info_text}
 
-    	$dirflow_tree bindText  <Double-ButtonPress-1> {Apol_Analysis_dirflow::treeSelect \
-		$Apol_Analysis_dirflow::dirflow_tree $Apol_Analysis_dirflow::dirflow_info_text}
+    	$dirflow_tree bindText  <Double-ButtonPress-1> {
+    		Apol_Analysis_dirflow::treeSelect \
+		$Apol_Analysis_dirflow::dirflow_tree \
+		$Apol_Analysis_dirflow::dirflow_info_text}
     
 	return $dirflow_tree
 }
 
-proc Apol_Analysis_dirflow::get_selected_objects { } {
+proc Apol_Analysis_dirflow::get_unselected_objects { } {
         variable list_objs
         variable objects_sel
 
-        set selected_objects ""
+        set unselected_objects ""
         set len [$list_objs size]
         
         if {$objects_sel} {
             for {set i 0} {$i < $len} {incr i} {
-	        if { [$list_objs selection includes $i] } {
-		    set selected_objects [lappend selected_objects [$list_objs get $i]]
+	        if {![$list_objs selection includes $i]} {
+		    set unselected_objects [lappend unselected_objects [$list_objs get $i]]
 	        }
             }
         }
-        return $selected_objects
+        return $unselected_objects
 }
 
 
@@ -1141,7 +1181,7 @@ proc Apol_Analysis_dirflow::change_types_list { } {
 		set rt [catch {set attrib_typesList [apol_GetAttribTypesList $display_attribute]} err]	
 		if {$rt != 0} {
 			tk_messageBox -icon error -type ok -title "Error" -message "$err"
-			return -code error
+			return
 		} 
 		set attrib_typesList [lsort $attrib_typesList]
 		set idx [lsearch -exact $attrib_typesList "self"]
@@ -1256,7 +1296,8 @@ proc Apol_Analysis_dirflow::create_options { options_frame } {
         set endtype_frame [frame $right.endtype_frame]
 
 	# Information Flow Entry frames
-	set lbl_start_type [Label $start_frame.lbl_start_type -text "Starting type:"]
+	set lbl_start_type [Label $start_frame.lbl_start_type \
+		-text "Starting type:"]
     	set combo_start [ComboBox $start_frame.combo_start \
     		-helptext "You must choose a starting type for information flow" \
 		-editable 1 \
@@ -1264,29 +1305,35 @@ proc Apol_Analysis_dirflow::create_options { options_frame } {
 		-entrybg white \
 		-exportselection 0]  
 
-        set lbl_flowtype [Label $flowtype_frame.lbl_flowtype -text "Flow direction:"]
+        set lbl_flowtype [Label $flowtype_frame.lbl_flowtype \
+        	-text "Flow direction:"]
 
-        set in_button [checkbutton $ckbttn_frame.in_button -text "In" \
+        set in_button [checkbutton $ckbttn_frame.in_button \
+        	-text "In" \
 		-variable Apol_Analysis_dirflow::in_button_sel \
 		-offvalue 0 -onvalue 1 \
 		-command { Apol_Analysis_dirflow::in_button_press }]
 
-        set out_button [checkbutton $ckbttn_frame.out_button -text "Out" \
+        set out_button [checkbutton $ckbttn_frame.out_button \
+        	-text "Out" \
 		-variable Apol_Analysis_dirflow::out_button_sel \
 		-offvalue 0 -onvalue 1 \
 		-command { Apol_Analysis_dirflow::out_button_press }]
 
-        set either_button [checkbutton $ckbttn_frame.either_button -text "Either" \
+        set either_button [checkbutton $ckbttn_frame.either_button \
+        	-text "Either" \
 		-variable Apol_Analysis_dirflow::either_button_sel \
 		-offvalue 0 -onvalue 1 \
 		-command { Apol_Analysis_dirflow::either_button_press }]
 
-        set both_button [checkbutton $ckbttn_frame.both_button -text "Both" \
+        set both_button [checkbutton $ckbttn_frame.both_button \
+        	-text "Both" \
 		-variable Apol_Analysis_dirflow::both_button_sel \
 		-offvalue 0 -onvalue 1 \
 		-command { Apol_Analysis_dirflow::both_button_press }]
 
-        set cb_attrib [checkbutton $attrib_frame.cb_attrib -text "Select starting type using attrib:" \
+        set cb_attrib [checkbutton $attrib_frame.cb_attrib \
+        	-text "Filter starting types to select using attribute:" \
 		-variable Apol_Analysis_dirflow::display_attrib_sel \
 		-offvalue 0 -onvalue 1 \
 		-command { Apol_Analysis_dirflow::config_attrib_comboBox_state }]
@@ -1296,12 +1343,15 @@ proc Apol_Analysis_dirflow::create_options { options_frame } {
     		-modifycmd { Apol_Analysis_dirflow::change_types_list} \
 		-exportselection 0] 
 
-        set clear_all_bttn [button $bttns_frame.clear_all_bttn -text "Clear All" \
+        set clear_all_bttn [button $bttns_frame.clear_all_bttn \
+        	-text "Clear All" \
 		-command {Apol_Analysis_dirflow::clear_all_button_press} ]
-        set select_all_bttn [button $bttns_frame.select_all_bttn -text "Select All" \
+        set select_all_bttn [button $bttns_frame.select_all_bttn \
+        	-text "Select All" \
 		-command {Apol_Analysis_dirflow::select_all_button_press} ]
 
-        set cb_endtype [checkbutton $endtype_frame.cb_endtype -text "Find end types using regular expression:" \
+        set cb_endtype [checkbutton $endtype_frame.cb_endtype \
+        	-text "Find end types using regular expression:" \
 		-variable Apol_Analysis_dirflow::endtype_sel \
 		-offvalue 0 -onvalue 1 -justify left -wraplength 150 \
 		-command { Apol_Analysis_dirflow::config_endtype_state }]
@@ -1312,16 +1362,18 @@ proc Apol_Analysis_dirflow::create_options { options_frame } {
 		-textvariable Apol_Analysis_dirflow::end_type \
 		-exportselection 0] 
 
-        set cb_objects [checkbutton $objcl_frame.cb_objects -text "Filter results by object class:" \
+        set cb_objects [checkbutton $objcl_frame.cb_objects \
+        	-text "Only include rules with selected object classes:" \
 		-variable Apol_Analysis_dirflow::objects_sel \
 		-offvalue 0 -onvalue 1 \
 		-justify left \
 		-command {Apol_Analysis_dirflow::config_objects_list_state }]
 
         set sw_objs [ScrolledWindow $objcl_frame.sw_objs -auto both]
-        set list_objs [listbox [$sw_objs getframe].list_objs -height 7 -highlightthickness 0 \
-		      -selectmode multiple \
-		      -exportselection 0] 
+        set list_objs [listbox [$sw_objs getframe].list_objs -height 7 \
+        	-highlightthickness 0 \
+		-selectmode multiple \
+		-exportselection 0] 
         $sw_objs setwidget $list_objs
 
         # pack all the widgets
