@@ -2,10 +2,7 @@
  * see file 'COPYING' for use and warranty information */
 
 /* 
- * Author: mayerf@tresys.com
- * Modified by: don.patterson@tresys.com (6-17-2003)
- * Modified by: kmacmillan@tresys.com (7-18-2003) - added
- *   information flow analysis.
+ * Author: kmacmillan@tresys.com
  * Modified by: mayerf@tresys.com (Apr 2004) - separated information
  *   flow from main analysis.c file, and added noflow/onlyflow batch
  *   capabilitiy.
@@ -75,7 +72,6 @@ typedef struct iflow_graph {
 } iflow_graph_t;
 
 /* iflow_query_t */
-
 iflow_query_t *iflow_query_create(void)
 {
 	iflow_query_t* q = (iflow_query_t*)malloc(sizeof(iflow_query_t));
@@ -90,7 +86,7 @@ iflow_query_t *iflow_query_create(void)
 	return q;
 }
 
-static int iflow_obj_options_copy(analysis_obj_options_t *dest, analysis_obj_options_t *src)
+static int iflow_obj_options_copy(obj_perm_set_t *dest, obj_perm_set_t *src)
 {
         dest->obj_class = src->obj_class;
         dest->num_perms = src->num_perms;
@@ -127,13 +123,13 @@ static int iflow_query_copy(iflow_query_t *dest, iflow_query_t *src)
 
         if (src->num_obj_options) {
                 assert(src->obj_options);
-                dest->obj_options = (analysis_obj_options_t*)malloc(sizeof(analysis_obj_options_t) * 
+                dest->obj_options = (obj_perm_set_t*)malloc(sizeof(obj_perm_set_t) * 
                                                                  src->num_obj_options);
                 if (!dest->obj_options) {
                         fprintf(stderr, "Memory error\n");
                         return -1;
                 }
-                memset(dest->obj_options, 0, sizeof(analysis_obj_options_t) * src->num_obj_options);
+                memset(dest->obj_options, 0, sizeof(obj_perm_set_t) * src->num_obj_options);
                 for (i = 0; i < src->num_obj_options; i++) {
                         if (iflow_obj_options_copy(dest->obj_options + i, src->obj_options + i))
                                 return -1;
@@ -161,51 +157,35 @@ void iflow_query_destroy(iflow_query_t *q)
 	free(q);
 }
 
+/* Object class filter macros.
+ *	Transitive information flow - if perms is non-NULL then only those 
+ *	permissions are ignored, otherwise the entire object class is ignored. 
+ */
 int iflow_query_add_obj_class(iflow_query_t *q, int obj_class)
 {
-	return analysis_query_add_obj_class(&q->obj_options, &q->num_obj_options, obj_class);
+	return apol_add_class_to_obj_perm_set_list(&q->obj_options, &q->num_obj_options, obj_class);
 
 }
 
 int iflow_query_add_obj_class_perm(iflow_query_t *q, int obj_class, int perm)
 {
-	return analysis_query_add_obj_class_perm(&q->obj_options, &q->num_obj_options, obj_class, perm);
+	return apol_add_perm_to_obj_perm_set_list(&q->obj_options, &q->num_obj_options, obj_class, perm);
 }
 
 int iflow_query_add_end_type(iflow_query_t *q, int end_type)
 {
-	return analysis_query_add_end_type(&q->end_types, &q->num_end_types, end_type);
+	return policy_query_add_type(&q->end_types, &q->num_end_types, end_type);
 }
 
 int iflow_query_add_type(iflow_query_t *q, int type)
 {
-	bool_t add = FALSE;
-
-	assert(q);
-	/* we can't do anymore checking without the policy */
-	if (type < 0) {
-		fprintf(stderr, "end type must be 0 or greater\n");
-		return -1;
-	}
-
-	if (q->types) {
-		if (find_int_in_array(type, q->types,
-				      q->num_types) < 0) {
-			add = TRUE;
-		}
-	} else {
-		add = TRUE;
-	}
-	if (add)
-		if (add_i_to_a(type, &q->num_types, &q->types) < 0)
-			return -1;
-	return 0;
+	return policy_query_add_type(&q->types, &q->num_types, type);
 }
 
 /*
  * Check that the iflow_obj_option_t is valid for the graph/policy.
  */
-bool_t iflow_obj_option_is_valid(analysis_obj_options_t *o, policy_t *policy)
+bool_t iflow_obj_option_is_valid(obj_perm_set_t *o, policy_t *policy)
 {
 	int i;
 
@@ -579,7 +559,7 @@ static int add_edges(iflow_graph_t* g, int obj_class, int rule_idx, bool_t found
 	bool_t all_tgt_types = FALSE;
 	int cur_tgt_type;
 	int num_tgt_types = 0;
-	int* tgt_types = NULL;
+	int *tgt_types = NULL;
 
 	av_item_t* rule;
 
@@ -599,10 +579,16 @@ static int add_edges(iflow_graph_t* g, int obj_class, int rule_idx, bool_t found
 		all_tgt_types = TRUE;
 	
 	for (i = 0; i < num_src_types; i++) {
-		if (all_src_types)
+		if (all_src_types) {
+			/* skip self */
+			if (i == 0)
+				continue;
 			cur_src_type = i;
-		else
+		} else {
 			cur_src_type = src_types[i];
+		}
+		/* self should never be a src type */
+		assert(cur_src_type);
 
 		if (g->query->num_types) {
 			bool_t filter_type = FALSE;
@@ -625,10 +611,16 @@ static int add_edges(iflow_graph_t* g, int obj_class, int rule_idx, bool_t found
 		for (j = 0; j < num_tgt_types; j++) {
 			int edge;
 			
-			if (all_tgt_types)
+			if (all_tgt_types) {
 				cur_tgt_type = j;
-			else
+				if (j == 0)
+					continue;
+			} else if (tgt_types[j]==0) {
+				/* idx 0 is self */
+				cur_tgt_type = cur_src_type;
+			} else {
 				cur_tgt_type = tgt_types[j];
+			}
 			
 			if (g->query->num_types) {
 				bool_t filter_type = FALSE;
@@ -658,7 +650,6 @@ static int add_edges(iflow_graph_t* g, int obj_class, int rule_idx, bool_t found
 				if (add_i_to_a(rule_idx, &g->edges[edge].num_rules,
 					       &g->edges[edge].rules) != 0) {
 					fprintf(stderr, "Could not add rule!\n");
-					return -1;
 				}
 			}
 			if (found_write) {
@@ -667,13 +658,12 @@ static int add_edges(iflow_graph_t* g, int obj_class, int rule_idx, bool_t found
 					fprintf(stderr, "Could not add edge!\n");
 					return -1;
 				}
+				
 				if (add_i_to_a(rule_idx, &g->edges[edge].num_rules,
 					       &g->edges[edge].rules) != 0) {
 					fprintf(stderr, "Could not add rule!\n");
-					return -1;
 				}
 			}
-			
 		}
 	}
 	if (!all_src_types) {
@@ -694,6 +684,7 @@ iflow_graph_t *iflow_graph_create(policy_t* policy, iflow_query_t *q)
 	unsigned char map;
 	iflow_graph_t* g;
 	bool_t perm_error = FALSE;
+        int max_len = PERMMAP_MAX_WEIGHT - q->min_weight + 1;
 
 	assert(policy && q);
 
@@ -782,7 +773,7 @@ iflow_graph_t *iflow_graph_create(policy_t* policy, iflow_query_t *q)
 			}
 			
 			read_len = write_len = INT_MAX;
-			for (k = 0; k < num_perms; k++) {				
+			for (k = 0; k < num_perms; k++) {
 				cur_perm = perms[k];
 				/* Check to see if we should ignore this permission */
 				if (cur_obj_options >= 0) {
@@ -815,14 +806,16 @@ iflow_graph_t *iflow_graph_create(policy_t* policy, iflow_query_t *q)
 					continue;
 				}
 				if (map & PERMMAP_READ) {
-					found_read = TRUE;
-					if (len < read_len)
+                                        if (len < read_len && len <= max_len) {
+                                                found_read = TRUE;
 						read_len = len;
+                                        }
 				}
 				if (map & PERMMAP_WRITE) {
-					found_write = TRUE;
-					if (len < write_len)
+					if (len < write_len && len <= max_len) {
+                                                found_write = TRUE;
 						write_len = len;
+                                        }
 				}
 			}
 			if (all_perms)
@@ -1497,6 +1490,8 @@ iflow_transitive_t *iflow_transitive_flows(policy_t *policy, iflow_query_t *q)
 		goto out;
 	}
 	
+	a->start_type = q->start_type;
+	
 	for (i = 0; i < num_nodes; i++) {
 		if (iflow_graph_shortest_path(g, nodes[i], a, q) != 0)
 			goto err;
@@ -1842,5 +1837,3 @@ void iflow_find_paths_abort(void *state)
 	free(s);
 	iflow_transitive_destroy(s->a);
 }
-
-
