@@ -60,7 +60,7 @@ extern void yyrestart(FILE *);
 extern unsigned int pass;
 extern int yydebug;
 
-/* returns an error string based on a return error from seuser_label_home_dir() */
+/* returns an error string based on a return error */
 const char* find_default_policy_file_strerr(int err)
 {
 	switch(err) {
@@ -450,7 +450,7 @@ static int read_policy(policy_t *policy)
 	#define OPEN_PERM_CHECK_18 "nlmsg_write"
 	rt = get_perm_idx(OPEN_PERM_CHECK_18, policy);
 	if(rt >= 0) { /* permission does exists; at least a version 18 policy */
-		rt = set_policy_version(POL_VER_18_19, policy);
+		rt = set_policy_version(POL_VER_18_20, policy);
 		if(rt < 0) {
 			fprintf(stderr, "error setting policy version to version 18.\n");
 			return -1;
@@ -468,7 +468,7 @@ unsigned int validate_policy_options(unsigned int options)
 	opts |= (POLOPT_COND_BOOLS|POLOPT_COND_EXPR);
 
 	/* NOTE: The order of these is important */
-	if(POLOPT_COND_TE_RULES)
+	if(POLOPT_COND_TE_RULES & opts)
 		opts |= POLOPT_TYPES|POLOPT_OBJECTS;	
 	if(POLOPT_TE_RULES & opts)
 		opts |= (POLOPT_OBJECTS|POLOPT_TYPES);
@@ -482,8 +482,14 @@ unsigned int validate_policy_options(unsigned int options)
 		opts |= POLOPT_TYPES;
 	if(POLOPT_INITIAL_SIDS & opts)
 		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_USERS);
+	if(POLOPT_OCONTEXT & opts)
+		opts |= (POLOPT_TYPES|POLOPT_ROLES|POLOPT_USERS);
 	if(POLOPT_OBJECTS & opts)
 		opts |= POLOPT_OBJECTS;
+	if(POLOPT_CONSTRAIN & opts)
+		opts |= POLOPT_SYMBOLS;
+	if(POLOPT_RANGETRANS & opts)
+		opts |= (POLOPT_MLS_COMP|POLOPT_TYPES);
 	
 	return opts;
 }
