@@ -94,10 +94,11 @@ int init_teq_results(teq_results_t *r);
 int init_teq_query(teq_query_t *q);
 
 int match_rbac_rules(int idx, int type, unsigned char whichlist, bool_t do_indirect,bool_t tgt_is_role, rbac_bool_t *b, policy_t *policy);
-int match_rbac_roles(int idx, int type, unsigned char whichlist, bool_t	do_indirect, bool_t tgt_is_role, rbac_bool_t *b, policy_t *policy);
+int match_rbac_roles(int idx, int type, unsigned char whichlist, bool_t	do_indirect, bool_t tgt_is_role, rbac_bool_t *b,int *num_matched, policy_t *policy);
 int match_te_rules(bool_t allow_regex, regex_t *preg, int ta_opt,int idx, int idx_type, bool_t include_audit, unsigned char whichlists,	
 	bool_t do_indirect, bool_t only_enabled, rules_bool_t *rules_b, policy_t *policy);
 int search_te_rules(teq_query_t *q, teq_results_t *r, policy_t *policy);
+bool_t match_rbac_role_ta(int rs_idx,int ta_idx, int *rt_idx,policy_t *policy);
 
 int search_conditional_expressions(bool_t use_bool, char *bool, bool_t allow_regex, bool_t *exprs_b, char **error_msg, policy_t *policy);
 
