@@ -1,4 +1,4 @@
- /* Copyright (C) 2001-2003 Tresys Technology, LLC
+/* Copyright (C) 2001-2003 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /* 
@@ -18,7 +18,6 @@
 #include <assert.h>
 #include <unistd.h>
 
-
 /* convert from string to intger "bool" */
 bool_t getbool(const char *str) 
 {
@@ -567,7 +566,7 @@ const char* libapol_get_version(void)
  */
 char* find_file(const char *file_name)
 {
-	char *file, *var = NULL, *dir = NULL;
+	char *file = NULL, *var = NULL, *dir = NULL;
 	int filesz;
 	int rt;
 	
