@@ -15,11 +15,14 @@
 
 
 #include "auditlog.h"
-#define	PARSE_RET_SUCCESS		0x00000000	/* success, no warnings nor errors */
-#define PARSE_RET_MEMORY_ERROR		0x00000001	/* general error */
-#define PARSE_RET_EOF_ERROR            	0x00000002	/* file was eof */
-#define PARSE_RET_NO_SELINUX_ERROR   	0x00000004	/* no selinux messages found */
-#define PARSE_RET_INVALID_MSG_WARN	0x00000008	/* invalid message, but added to audit log anyway */
+#define	PARSE_RET_SUCCESS		0x00000001	/* success, no warnings nor errors */
+#define PARSE_RET_MEMORY_ERROR		0x00000002	/* general error */
+#define PARSE_RET_EOF_ERROR            	0x00000004	/* file was eof */
+#define PARSE_RET_NO_SELINUX_ERROR   	0x00000008	/* no selinux messages found */
+#define PARSE_RET_INVALID_MSG_WARN	0x00000010	/* invalid message, but added to audit log anyway */
+#define PARSE_REACHED_END_OF_MSG	0x00000020	/* we reached the end of the message before gathering all information */
+#define LOAD_POLICY_FALSE_POS		0x00000040	/* indicates that the message is not a load message although has 'security:' string */
+#define LOAD_POLICY_NEXT_LINE 		0x00000080	/* indicates that we've parsed the first line of a load message */
 
 #define PARSE_MEMORY_ERROR_MSG "Memory error while parsing the log!"
 #define PARSE_NO_SELINUX_ERROR_MSG "No SELinux messages found in log!"
