@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Tresys Technology, LLC
+/* Copyright (C) 2003-2005 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -8,6 +8,6 @@
 #ifndef QUERY_WINDOW_H
 #define QUERY_WINDOW_H
 
-int query_window_create(void);
+int query_window_create(int *tree_item_idx);
 
 #endif
